import os, math

import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from matplotlib.patches import Rectangle

import scipy.optimize  as opt
import scipy.integrate as integrate
from scipy.integrate import quad
from scipy.stats import gaussian_kde
from scipy import stats
from scipy.optimize import curve_fit

import seaborn as sns
from mpl_toolkits.axes_grid1 import make_axes_locatable

TableGoossens = pd.read_csv('M14k2calc.csv')

# Define some constant values
# Gravity constant
G = 6.6743e-11
# Radius Mercury
ra= 2440.e3
# Gas constant
r = 8.3144
# Define the gravity function
def g(r):
    def mr(ra,rb):
        return 4*np.pi*integrate.quad(lambda rx:rho(rx)*rx**2, ra, rb)[0]

    if (r<=ricb):
        m=mr(0,r)
    elif (ricb<r<=rcmb):
        m=mr(0,ricb)+mr(ricb,r)
    elif (rcmb<r<=rmcb):
        m=mr(0,ricb)+mr(ricb,rcmb)+mr(rcmb,r)
    else:
        m=mr(0,ricb)+mr(ricb,rcmb)+mr(rcmb,rmcb)+mr(rmcb,r)

    if(r<1.e3):
        return 4/3*np.pi*G*r*rho(r)
    else:
        return m*G/r**2
# Define the pressure function
def p(r):
    def pz(za,zb):
        return integrate.quad(lambda z:rho(ra-z)*g(ra-z), za, zb)[0]

    if (rmcb<r<=ra):
        return pz(0,ra-r)
    elif (rcmb<r<=rmcb):
        return pz(0,ra-rmcb)+pz(ra-rmcb,ra-r)
    elif (ricb<r<=rcmb):
        return pz(0,ra-rmcb)+pz(ra-rmcb,ra-rcmb)+pz(ra-rcmb,ra-r)
    else:
        return pz(0,ra-rmcb)+pz(ra-rmcb,ra-rcmb)+pz(ra-rcmb,ra-ricb)+pz(ra-ricb,ra-r)
# Define the temperature of the magma ocean at CMB in a sulfur-free system [K]
def TliqSfree(PGPa, S, dTdS):
    return [2092 + 3.40*(PGPa) + 16.88*pow(PGPa,2)  - 2.43*pow(PGPa,3) + 0.14*pow(PGPa,4), 2037 + 53.4*(PGPa) + 6.88*pow(PGPa,2)  - 1.88*pow(PGPa,3) + 0.14*pow(PGPa,4) - dTdS, 2037 + 53.4*(PGPa) + 6.88*pow(PGPa,2)  - 1.88*pow(PGPa,3) + 0.14*pow(PGPa,4) - dTdS - 60]
def TliqSfreeSi4(PGPa, S, dTdS):
    return [2106 - 7.46*(PGPa) + 25.93*pow(PGPa,2)  - 4.47*pow(PGPa,3) + 0.274*pow(PGPa,4), 2106 - 7.46*(PGPa) + 25.93*pow(PGPa,2)  - 4.47*pow(PGPa,3) + 0.274*pow(PGPa,4) - dTdS, 2106 - 7.46*(PGPa) + 25.93*pow(PGPa,2)  - 4.47*pow(PGPa,3) + 0.274*pow(PGPa,4) - dTdS - 60]
def TliqSfreeSi8(PGPa, S, dTdS):
    return [2124 + 5.58*(PGPa) + 14.28*pow(PGPa,2)  - 2.23*pow(PGPa,3) + 0.142*pow(PGPa,4), 2124 + 5.58*(PGPa) + 14.28*pow(PGPa,2)  - 2.23*pow(PGPa,3) + 0.142*pow(PGPa,4) - dTdS, 2124 + 5.58*(PGPa) + 14.28*pow(PGPa,2)  - 2.23*pow(PGPa,3) + 0.142*pow(PGPa,4) - dTdS - 60]
def TliqSfreeSi15(PGPa, S, dTdS):
    return [2144 + 40.54*(PGPa) + 5.18*pow(PGPa,2)  - 2.80*pow(PGPa,3) + 0.262*pow(PGPa,4), 2144 + 40.54*(PGPa) + 5.18*pow(PGPa,2)  - 2.80*pow(PGPa,3) + 0.262*pow(PGPa,4) - dTdS, 2144 + 40.54*(PGPa) + 5.18*pow(PGPa,2)  - 2.80*pow(PGPa,3) + 0.262*pow(PGPa,4) - dTdS - 60]

# Thermodynamic model of Day 2012 for graphite-diamnond transition
def CpDay(a,b,c,d,e,f,g,h,tref, t):
    CpDay = (a*t + 0.5*b*t**2 - c*t**-1  + 2*d*t**0.5 + (1/3)*e*t**3 -0.5* f*t**-2 - (1/3)*g*t**-3 + 0.25*h*t**4) - (a*tref + 0.5*b*tref**2 - c*tref**-1  + 2*d*tref**0.5 + (1/3)*e*tref**3 -0.5* f*tref**-2 - (1/3)*g*tref**-3 + 0.25*h*tref**4)
    return CpDay/1000

def CpTDay(a,b,c,d,e,f,g,h,tref, t):
    CpTDay = (a*np.log(t) + b*t- 0.5*c*t**-2 -2*d*t**-0.5 + 0.5*e*t**2 - (1/3)*f*t**-3 - (1/4)*g*t**-4 + (1/3)*h*t**3) - (a*np.log(tref) + b*tref - 0.5*c*tref**-2 -2*d*tref**-0.5 + 0.5*e*tref**2 - (1/3)*f*tref**-3 - (1/4)*g*tref**-4 + (1/3)*h*tref**3)
    return CpTDay/1000

def vdpgrDay(B0, Bpr, t, tref, v1, pkbar, n, a):
    v1t = v1*(1 + a*(t-tref) - 20*a*(t**0.5-tref**0.5))
    B = B0 + Bpr*(t-tref)
    v = v1t*(1 + (n*pkbar)/B)**(-1/n)
    vdP = (v*B/(n-1))*(1 + (n*pkbar/B)**(1-1/n))
    #return vdP
    return v

def vdpdiaDay(x1, x2, x3, o1, o2, o3, B0, Bpr, t, tref, v1, pkbar, n):
    x1 = x1*o1/(np.exp(o1/t)-1)
    x2 = x2*o2/(np.exp(o2/t)-1)
    x3 = x3*o3/(np.exp(o3/t)-1)
    v1t = v1*np.exp(3*(x1 + x2 + x3))
    B = B0 + Bpr*(t-tref)
    v = v1t*(1 + (n*pkbar)/B)**(-1/n)
    vdP = (v*B/(n-1))*(1 + (n*pkbar/B)**(1-1/n))
    #return vdP
    return v

def graphitediamondtransitionDay(pkbar, hgr, sgr, hdiamond, sdiamond, t):
    Ggr = hgr + CpDay(a = 2.430e1, b = 9.446e-4, c = -5.125e6, d = 0, e = 0, f = 1.586e9, g = -1.440e11, h = 0, tref = 298.15, t = t) - t*(sgr + CpTDay(a = 2.430e1, b = 9.446e-4, c = -5.125e6, d=0, e=0, f = 1.586e9, g = -1.440e11, h=0, tref = 298.15, t = t))
    #Ggr = hgr + CpDay(a = 6.086e1, b = -1.024e-2, c = 7.13e5, d = -9.92e2, e = 1.66e-6, f = 0, g = 0, h = 0, tref = 298.15, t = t) - t*(sgr + CpTDay(a = 6.086e1, b = -1.024e-2, c = 7.13e5, d = -9.92e2, e = 1.66e-6, f = 0, g = 0, h = 0, tref = 298.15, t = t))
    #Ggr = hgr + CpDay(a = 4.824e0, b = 2.86e-2, c = -3.25e5, d = 0, e = -1.31e-5, f = 0, g = 0, h = 2.27e-9, tref = 298.15, t = t) - t*(sgr + CpTDay(a = 6.086e1, b = -1.024e-2, c = 7.13e5, d = -9.92e2, e = 1.66e-6, f = 0, g = 0, h = 0, tref = 298.15, t = t))
    vdpgr = quad(lambda v: vdpgrDay(B0 = 338, Bpr = -0.07, t = t, tref = 298, v1 = 0.530, pkbar = pkbar, n = 8.9, a = 4.84e-5),  0, pkbar)[0]
    #vdpgr = vdpgrDay(B0 = 338, Bpr = -0.07, t = t, tref = 298, v1 = 0.530, pkbar = pkbar, n = 8.9, a = 4.84e-5)
    ggr = Ggr + vdpgr*0.5475*pow(t,0.0874)

    Gdia = hdiamond + CpDay(a = 2.431e1, b = 9.446e-4, c = -5.396e6, d=0, e=0, f = 1.566e9, g = -1.332e11, h=0, tref = 298.15, t = t) - t*(sdiamond + CpTDay(a = 2.431e1, b = 9.446e-4, c = -5.396e6, d=0, e=0, f = 1.566e9, g = -1.332e11, h=0, tref = 298.15, t = t))
    #Gdia = hdiamond + CpDay(a = 9.84e1, b = -3.65e-2, c = 1.22e6, d=-1.66e3, e=1.098e-5, f = 0, g = 0, h=0, tref = 298.15, t = t) - t*(sdiamond + CpTDay(a = 9.84e1, b = -3.65e-2, c = 1.22e6, d=-1.66e3, e=1.098e-5, f = 0, g = 0, h=0, tref = 298.15, t = t))
    vdpdia = quad(lambda v: vdpdiaDay(x1 = 4.36e-8, x2 = 1.57e-6, x3 = 4.2559e-6, o1 = 200, o2 = 880, o3 = 2138, B0 = 4460, Bpr = -0.3, t = t, tref = 298, v1 = 0.341, pkbar = pkbar, n = 3.2), 0, pkbar)[0]
    #vdpdia = vdpdiaDay(x1 = 4.36e-8, x2 = 1.57e-6, x3 = 4.255e-6, o1 = 200, o2 = 880, o3 = 2138, B0 = 4460, Bpr = -0.3, t = t, tref = 298, v1 = 0.341, pkbar = pkbar, n = 3.2)
    gdia = Gdia + vdpdia
    return ggr - gdia

def graphitediamondtransitionDayt(t, pkbar, hgr, sgr, hdiamond, sdiamond):
    Ggr = hgr + CpDay(a = 2.430e1, b = 9.446e-4, c = -5.125e6, d = 0, e = 0, f = 1.586e9, g = -1.440e11, h = 0, tref = 298.15, t = t) - t*(sgr + CpTDay(a = 2.430e1, b = 9.446e-4, c = -5.125e6, d=0, e=0, f = 1.586e9, g = -1.440e11, h=0, tref = 298.15, t = t))
    #Ggr = hgr + CpDay(a = 6.086e1, b = -1.024e-2, c = 7.13e5, d = -9.92e2, e = 1.66e-6, f = 0, g = 0, h = 0, tref = 298.15, t = t) - t*(sgr + CpTDay(a = 6.086e1, b = -1.024e-2, c = 7.13e5, d = -9.92e2, e = 1.66e-6, f = 0, g = 0, h = 0, tref = 298.15, t = t))
    #Ggr = hgr + CpDay(a = 4.824e0, b = 2.86e-2, c = -3.25e5, d = 0, e = -1.31e-5, f = 0, g = 0, h = 2.27e-9, tref = 298.15, t = t) - t*(sgr + CpTDay(a = 6.086e1, b = -1.024e-2, c = 7.13e5, d = -9.92e2, e = 1.66e-6, f = 0, g = 0, h = 0, tref = 298.15, t = t))
    vdpgr = quad(lambda v: vdpgrDay(B0 = 338, Bpr = -0.07, t = t, tref = 298, v1 = 0.530, pkbar = pkbar, n = 8.9, a = 4.84e-5),  0, pkbar)[0]
    #vdpgr = vdpgrDay(B0 = 338, Bpr = -0.07, t = t, tref = 298, v1 = 0.530, pkbar = pkbar, n = 8.9, a = 4.84e-5)
    ggr = Ggr + vdpgr*0.5475*pow(t,0.0874)

    Gdia = hdiamond + CpDay(a = 2.431e1, b = 9.446e-4, c = -5.396e6, d=0, e=0, f = 1.566e9, g = -1.332e11, h=0, tref = 298.15, t = t) - t*(sdiamond + CpTDay(a = 2.431e1, b = 9.446e-4, c = -5.396e6, d=0, e=0, f = 1.566e9, g = -1.332e11, h=0, tref = 298.15, t = t))
    #Gdia = hdiamond + CpDay(a = 9.84e1, b = -3.65e-2, c = 1.22e6, d=-1.66e3, e=1.098e-5, f = 0, g = 0, h=0, tref = 298.15, t = t) - t*(sdiamond + CpTDay(a = 9.84e1, b = -3.65e-2, c = 1.22e6, d=-1.66e3, e=1.098e-5, f = 0, g = 0, h=0, tref = 298.15, t = t))
    vdpdia = quad(lambda v: vdpdiaDay(x1 = 4.36e-8, x2 = 1.57e-6, x3 = 4.2559e-6, o1 = 200, o2 = 880, o3 = 2138, B0 = 4460, Bpr = -0.3, t = t, tref = 298, v1 = 0.341, pkbar = pkbar, n = 3.2), 0, pkbar)[0]
    #vdpdia = vdpdiaDay(x1 = 4.36e-8, x2 = 1.57e-6, x3 = 4.255e-6, o1 = 200, o2 = 880, o3 = 2138, B0 = 4460, Bpr = -0.3, t = t, tref = 298, v1 = 0.341, pkbar = pkbar, n = 3.2)
    gdia = Gdia + vdpdia
    return ggr - gdia


# S content magma ocean wt%
S_array = np.array([7, 11])

for S in S_array:
    print('start calculation for S = ' + str(S) + ' wt.%')
    # [dT/dS]p
    dTdS = 59.2*pow(S,0.75)
    #for i in range(len(TableGoossens)):
    # Create matrices to record the data
    record = pd.DataFrame(columns=['Pressure CMB [GPa]', 'Tliq S-free', 'Tliq S-bearing', 'Tliq50', 'DiamondT_Sfree', 'DiamondT_S', 'DiamondT_50', 'TsatPcmb'])
    n = 100000
    records_list = []
    for i in range(n):
        if(i%10000==0): # Will report temperature of calculation every 100K
            print('calculating iteration' f' {i} of'  f' {n}')
        if i == n-1:
            print('calculation done')

    # Define the thicknesses of the different layers
        rmcb = ra - TableGoossens['cr_thickness'][i] # mantle-crust boundary
        rcmb = TableGoossens['outer_core_size'][i] # mantle-core boundary
        ricb = TableGoossens['inner_core_size'][i] # inner core - outer core boundary

    # Define the densities of the different layers
        def rho(r):
           if(r<=ricb):
               rho=TableGoossens['ave_ic_density'][i]
           elif(r<=rcmb):
               rho=TableGoossens['ave_oc_density'][i]
           elif(r<=rmcb):
               rho=TableGoossens['mantle_density'][i]
           else:
               rho=TableGoossens['crustal_density'][i]
           return rho

    # Calculate the pressure at the CMB
        pr = p(rcmb)/1e9

    # Calculate the liquidus temperature in a S-free system at the CMB
        Tliq_Sfree = TliqSfree(PGPa = pr, S = S, dTdS= dTdS)[0]
        Tliq_S = TliqSfree(PGPa = pr, S = S, dTdS = dTdS)[1]
        Tliq_50cryst = TliqSfree(PGPa = pr, S = S, dTdS = dTdS)[2]

    # Determine whether or not the magma ocean at CMB is within the stability field of diamond
        pgraphitediamond_P_Tliq_Sfree = lambda pkbar: graphitediamondtransitionDay(pkbar, t = Tliq_Sfree, hgr = 0, sgr = 5.74/1000, hdiamond = 1.872, sdiamond = 2.38/1000)
        pgraphitediamond_P_Tliq_S = lambda pkbar: graphitediamondtransitionDay(pkbar, t = Tliq_S, hgr = 0, sgr = 5.74/1000, hdiamond = 1.872, sdiamond = 2.38/1000)
        pgraphitediamond_P_Tliq_50cryst = lambda pkbar: graphitediamondtransitionDay(pkbar, t = Tliq_50cryst, hgr = 0, sgr = 5.74/1000, hdiamond = 1.872, sdiamond = 2.38/1000)
        pgraphitediamond_P_Tliq_Sfree = opt.fsolve(pgraphitediamond_P_Tliq_Sfree, 2)
        pgraphitediamond_P_Tliq_S = opt.fsolve(pgraphitediamond_P_Tliq_S, 2)
        pgraphitediamond_P_Tliq_50cryst = opt.fsolve(pgraphitediamond_P_Tliq_50cryst, 2)
        if pgraphitediamond_P_Tliq_Sfree*0.1 < pr:
            pgraphitediamond_P_Tliq_Sfree = 1
        else:
            pgraphitediamond_P_Tliq_Sfree = 0

        if pgraphitediamond_P_Tliq_S*0.1 < pr:
            pgraphitediamond_P_Tliq_S = 1
        else:
            pgraphitediamond_P_Tliq_S = 0

        if pgraphitediamond_P_Tliq_50cryst*0.1 < pr:
            pgraphitediamond_P_Tliq_50cryst = 1
        else:
            pgraphitediamond_P_Tliq_50cryst = 0

        pgraphitediamond_T_transition = lambda t: graphitediamondtransitionDayt(t, pkbar = pr*10, hgr = 0, sgr = 5.74/1000, hdiamond = 1.872, sdiamond = 2.38/1000)
        pgraphitediamond_T_transition = opt.brentq(pgraphitediamond_T_transition, 1000,2500)

    # # Record the results of the calculations
        records_list.append(pd.DataFrame({'Pressure CMB [GPa]': [pr], 'Tliq S-free': [Tliq_Sfree], 'Tliq S-bearing': [Tliq_S], 'Tliq50': [Tliq_50cryst], 'DiamondT_Sfree': [pgraphitediamond_P_Tliq_Sfree], 'DiamondT_S': [pgraphitediamond_P_Tliq_S], 'DiamondT_50': [pgraphitediamond_P_Tliq_50cryst], 'TsatPcmb': [pgraphitediamond_T_transition]}))

    # # Concatenate all individual records into one DataFrame
    record = pd.concat(records_list, ignore_index=True)
    record.to_csv (r'CMBCalculationsS'+str(S)+'aM14.csv', index = False, header=True)



# Load previously calculated P_cmb (done below in the code)
p_cmb_Goossens11 = pd.read_csv('CMBCalculationsS11aM14.csv')
p_cmb_Goossens7 = pd.read_csv('CMBCalculationsS7aM14.csv')

# Figure illustrating the effect of sulfur on the liquidus and graphite diamond transition

# Ranges of P, T considred
p_range = np.linspace(3, 8, 100)
t_range = np.linspace(1700, 2600, 100)
recp_tran = np.zeros(len(t_range))

# Calculate the liquidus curves for different S

for idx, ite in enumerate(t_range):
    pgraphitediamond_P_Tliq_Sfree = lambda pkbar: graphitediamondtransitionDay(pkbar, t = ite, hgr = 0, sgr = 5.74/1000, hdiamond = 1.872, sdiamond = 2.38/1000)
    pgraphitediamond_P_Tliq_Sfree = opt.fsolve(pgraphitediamond_P_Tliq_Sfree, 2)
    recp_tran[idx] = pgraphitediamond_P_Tliq_Sfree

# Plot the liquidus curves for different S
fig, axes = plt.subplots(1, 1, figsize=(6, 5), constrained_layout=True)
a1 = axes.plot(p_range, TliqSfreeSi8(p_range, S = 0, dTdS = 59.2*pow(0,0.75))[1], c = 'blue', label='S = 0 wt.%')
a1 = axes.plot(p_range, TliqSfreeSi8(p_range, S = 3, dTdS = 59.2*pow(3,0.75))[1], c = 'green', label='S = 3 wt.%')
a1 = axes.plot(p_range, TliqSfreeSi8(p_range, S = 6, dTdS = 59.2*pow(6,0.75))[1], c = 'yellow', label='S = 6 wt.%')
a1 = axes.plot(p_range, TliqSfreeSi8(p_range, S = 8, dTdS = 59.2*pow(8,0.75))[1], c = 'orange', label='S = 8 wt.%')
a1 = axes.plot(p_range, TliqSfreeSi8(p_range, S = 11, dTdS = 59.2*pow(11,0.75))[1], c = 'red', label='S = 11 wt.%')
a1 = axes.plot(recp_tran*1e-1, t_range, c = 'black', linestyle = '--', label = 'graphite-diamond')

# Calculate and plot density distributions for the temp of the liquidus
asins_p1 = axes.inset_axes(
            [0.02,0,0.1,1],
        )
asins_p1.set_facecolor("none")
asins_p1.set_yticks([])
asins_p1.set_xticks([])
asins_p1.spines['top'].set_visible(False)
asins_p1.spines['bottom'].set_visible(False)
asins_p1.spines['right'].set_visible(False)
asins_p1.spines['left'].set_visible(False)
asins_p1.set_ylim(1700,2600)

sns.kdeplot(
        y=p_cmb_Goossens11['Tliq S-bearing'],
        color='grey',
        ax=asins_p1,
        fill=True,
    )

sns.kdeplot(
        y=p_cmb_Goossens7['Tliq S-bearing'],
        color='red',
        ax=asins_p1,
        fill=True,
    )

# Calculate and plot density distributions for the pressure at the CMB
asins_p1 = axes.inset_axes(
            [0,0.88,1,.1],
        )
asins_p1.invert_yaxis()
asins_p1.set_facecolor("none")
asins_p1.set_yticks([])
asins_p1.set_xticks([])
asins_p1.spines['top'].set_visible(False)
asins_p1.spines['bottom'].set_visible(False)
asins_p1.spines['right'].set_visible(False)
asins_p1.spines['left'].set_visible(False)
asins_p1.set_xlim(3,8)

sns.kdeplot(
        x=p_cmb_Goossens11['Pressure CMB [GPa]'],
        color='grey',
        ax=asins_p1,
        fill=True,
    )

# Plot the thermodynamic model of graphite-diamond transitions
axes.set_xlabel('Pressure [GPa]',  fontsize=20)
axes.set_ylabel('Liquidus temperature [K]',  fontsize=20)
axes.grid(color='lightgrey', linestyle='--', linewidth=0.5, zorder=10000000)
axes.legend()
plt.savefig('gr_dmd.pdf')
